<?php

namespace App\Http\Controllers;

use App\Models\SupportCase;
use Illuminate\Http\Request;
use App\Models\Client;
use App\Models\Company;
use App\Models\User;
use Carbon\Carbon;

class SupportCaseController extends Controller
{
    public function __construct()
    {
        $this->middleware('permission:ver-soporte|crear-soporte|editar-soporte|borrar-soporte', ['only' => ['index']]);
        $this->middleware('permission:crear-soporte', ['only' => ['create', 'store']]);
        $this->middleware('permission:editar-soporte', ['only' => ['edit', 'update']]);
        $this->middleware('permission:borrar-soporte', ['only' => ['destroy']]);
    }

    public function index(Request $request)
    {
        $currentMonth = ucfirst(Carbon::now()->translatedFormat('F'));

        $cases = SupportCase::with(['company', 'client', 'consultant'])
            ->whereNull('deleted_at')
            ->orderBy('created_at', 'desc')
            ->get();

        $month = $request->input('month', now()->month); // Por defecto, el mes actual
        $year = $request->input('year', now()->year); // Por defecto, el año actual

        $supportCases = SupportCase::whereYear('request_date', $year)
            ->whereMonth('request_date', $month)
            ->get();

        return view('supports.index', compact('cases', 'supportCases', 'currentMonth', 'month', 'year'));
    }


    public function create()
    {
        $companies = Company::all();
        $clients = Client::all();
        $consultants = User::all();

        return view('supports.create', compact('companies', 'clients', 'consultants'));
    }


    public function store(Request $request)
    {
        $data = $request->validate([
            'request_date' => 'required|date',
            'type' => [
                'required',
                'string',
                'max:255',
                'regex:/^[a-zA-Z0-9áéíóúÁÉÍÓÚüÜñÑ\s\.\-\"\'\,]+$/',
            ],
            'company_id' => 'nullable|exists:companies,id',
            'client_id' => 'required|exists:clients,id',
            'issue_description' => [
                'required',
                'string',
                'max:255',
                'regex:/^[a-zA-Z0-9áéíóúÁÉÍÓÚüÜñÑ\s\.\-\"\'\,]+$/',
            ],
            'solution' => [
                'required',
                'string',
                'max:255',
                'regex:/^[a-zA-Z0-9áéíóúÁÉÍÓÚüÜñÑ\s\.\-\"\'\,]+$/',
            ],
            'resolution_date' => 'required|date',
            'start_time' => 'required|date_format:H:i',
            'end_time' => 'required|date_format:H:i',
            'opportunity' => [
                'nullable',
                'string',
                'max:255',
                'regex:/^[a-zA-Z0-9áéíóúÁÉÍÓÚüÜñÑ\s\.\-\"\'\,]+$/',
            ],
        ]);

        $data['type_count'] = 1;
        $data['company_count'] = 1;
        $data['consultant_case_count'] = 1;
        $data['consultant_id'] = $request->consultant_id ?? auth()->user()->id;

        SupportCase::create($data);

        return redirect()->route('supports.index')->with('success', 'El caso ha sido creado exitosamente.');
    }

    public function show() {}


    public function indicators()
    {
        $cases = SupportCase::selectRaw('type, EXTRACT(MONTH FROM request_date) as month, COUNT(*) as total')
            ->groupBy('type', 'month')
            ->orderBy('type')
            ->get();

        $indicators = [];
        $averages = [];
        $monthlyTotals = array_fill(1, 12, 0); // Inicializamos los totales mensuales

        // Organizar los casos por tipo y mes
        foreach ($cases as $case) {
            $indicators[$case->type][$case->month] = $case->total;

            // Sumar los casos por mes
            $monthlyTotals[$case->month] += $case->total;
        }

        // Calcular el promedio para cada tipo de caso
        foreach ($indicators as $type => $months) {
            $total = 0;
            $count = 0;

            // Sumar los casos por mes y contar los meses con datos
            for ($i = 1; $i <= 12; $i++) {
                if (isset($months[$i])) {
                    $total += $months[$i];
                    $count++;
                }
            }

            // Calcular el promedio y guardarlo
            if ($count > 0) {
                $averages[$type] = $total / $count;
            } else {
                $averages[$type] = 0; // Si no hay datos, el promedio es 0
            }
        }

        // Calcular el promedio general de todos los casos por mes
        $totalSum = array_sum($monthlyTotals); // Sumar todos los totales mensuales
        $generalAverage = $totalSum / 12; // Dividir entre 12 para obtener el promedio

        return view('supports.indicators', compact('indicators', 'averages', 'monthlyTotals', 'generalAverage'));
    }



    public function edit(SupportCase $supportCase, $id)
    {
        $supportCase = SupportCase::findOrFail($id);
        $companies = Company::all();
        $clients = Client::all();
        $consultants = User::all();

        return view('supports.edit', compact('supportCase', 'companies', 'clients', 'consultants'));
    }

    public function update(Request $request, $id)
    {
        $supportCase = SupportCase::findOrFail($id);

        $data = $request->validate([
            'request_date' => 'required|date',
            'type' => [
                'required',
                'string',
                'max:255',
                'regex:/^[a-zA-Z0-9áéíóúÁÉÍÓÚüÜñÑ\s\.\-\"\'\,]+$/',
            ],
            'company_id' => 'nullable|exists:companies,id',
            'client_id' => 'required|exists:clients,id',
            'issue_description' => [
                'required',
                'string',
                'max:255',
                'regex:/^[a-zA-Z0-9áéíóúÁÉÍÓÚüÜñÑ\s\.\-\"\'\,]+$/',
            ],
            'solution' => [
                'required',
                'string',
                'max:255',
                'regex:/^[a-zA-Z0-9áéíóúÁÉÍÓÚüÜñÑ\s\.\-\"\'\,]+$/',
            ],
            'resolution_date' => 'required|date',
            'start_time' => 'required',
            'end_time' => 'required',
            'opportunity' => [
                'nullable',
                'string',
                'max:255',
                'regex:/^[a-zA-Z0-9áéíóúÁÉÍÓÚüÜñÑ\s\.\-\"\'\,]+$/',
            ],
        ]);

        $supportCase->update([
            'request_date' => $request->request_date,
            'type' => $request->type,
            'company_id' => $request->company_id,
            'client_id' => $request->client_id,
            'issue_description' => $request->issue_description,
            'solution' => $request->solution,
            'consultant_id' => $request->consultant_id,
            'resolution_date' => $request->resolution_date,
            'start_time' => $request->start_time,
            'end_time' => $request->end_time,
            'opportunity' => $request->opportunity,
        ]);

        return redirect()->route('supports.index')->with('success', 'El caso ha sido actualizado correctamente.');
    }


    public function destroy(SupportCase $supportCase, $id)
    {
        $supportCase = SupportCase::findOrFail($id);
        $supportCase->delete();

        return redirect()->route('supports.index')->with('deleted', 'El caso se ha eliminado correctamente.');
    }
}
